package macros.rpc;

import haxe.macro.Context;
import haxe.macro.Expr;

using Lambda;

class RPC {
	macro static public function buildRPC():Array<Field> {
		var fields = Context.getBuildFields();

		var className = Context.getLocalClass().get().name + "RPC";
		var imports = Context.getLocalImports();

		var c = macro class $className extends macros.rpc.RPCBase {
			public function new(id:kernel.net.Package.NetworkID) {
				super(id, $v{className});
			}
		}

		for (field in fields) {
			if (field.meta == null)
				continue;
			if (field.meta.exists((i) -> i.name == "rpc") == false)
				continue;

			switch (field.kind) {
				case FFun(f):
					var argsExprs:Array<Expr> = [for (a in f.args) macro $i{a.name}];

					var convertedArgs = [];

					for (a in f.args) {
						a.type = Helper.resolveType(a.type, field.pos);
						convertedArgs.push(a);
					}

					c.fields.push({
						name: field.name,
						pos: field.pos,
						kind: FFun({
							args: convertedArgs,
							expr: macro {
								return cast this._performRequest($v{field.name}, $a{argsExprs});
							},
							ret: Helper.newPromise(Helper.resolveType(f.ret, field.pos)),
						}),
						access: [APublic],
						doc: null,
						meta: [],
					});
				default:
					Context.error("Only functions can be used for rpc", field.pos);
			}
		}

		haxe.macro.Context.defineType(c);
		return fields;
	}

	macro static public function generateRPCPackageHandle() {
		var proto = Context.getLocalClass().get().name + "RPC";
		var exprs:Array<Expr> = [];

		var fields = Context.getLocalClass().get().fields.get();

		for (field in fields) {
			if (field.meta == null)
				continue;
			if (!field.meta.has("rpc"))
				continue;

			switch (field.kind) {
				case FMethod(k):
					var funName = field.name;

					var exprsType = field.expr().t;

					switch (exprsType) {
						case TFun(args, ret):
							var callArgs = [for (k => v in args) macro pack.data.args[$v{k}]];

							if (Helper.isVoid(ret)) {
								exprs.push(macro {
									if (pack.data.func == $v{funName}) {
										this.$funName($a{callArgs});
										pack.respond(null);
									}
								});
							} else {
								exprs.push(macro {
									if (pack.data.func == $v{funName}) {
										pack.respond(this.$funName($a{callArgs}));
									}
								});
							}
						default:
							Context.error("Only functions can be used for rpc", field.pos);
					}
				default:
					Context.error("Only functions can be used for rpc", field.pos);
			}
		}

		return macro {
			kernel.net.Net.registerProto($v{proto}, (pack) -> {
				$a{exprs}
			});
		};
	}
}
